@@ -17,70 +17,83 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef CONTENT_SAO_HEADER
-#define CONTENT_SAO_HEADER
+#pragma once
 
-#include <util/numeric.h>
+#include "network/networkprotocol.h"
+#include "util/numeric.h"
 #include "serverobject.h"
 #include "itemgroup.h"
 #include "object_properties.h"
+#include "constants.h"
 
 class UnitSAO: public ServerActiveObject
 {
 public:
 	UnitSAO(ServerEnvironment *env, v3f pos);
-	virtual ~UnitSAO() {}
+	virtual ~UnitSAO() = default;
+
+	void setRotation(v3f rotation) { m_rotation = rotation; }
+	const v3f &getRotation() const { return m_rotation; }
+	v3f getRadRotation() { return m_rotation * core::DEGTORAD; }
 
-	virtual void setYaw(const float yaw) { m_yaw = yaw; }
-	float getYaw() const { return m_yaw; };
-	f32 getRadYaw() const { return m_yaw * core::DEGTORAD; }
 	// Deprecated
-	f32 getRadYawDep() const { return (m_yaw + 90.) * core::DEGTORAD; }
+	f32 getRadYawDep() const { return (m_rotation.Y + 90.) * core::DEGTORAD; }
 
-	s16 getHP() const { return m_hp; }
+	u16 getHP() const { return m_hp; }
 	// Use a function, if isDead can be defined by other conditions
 	bool isDead() const { return m_hp == 0; }
 
-	bool isAttached() const;
+	inline bool isAttached() const
+	{ return getParent(); }
+
 	void setArmorGroups(const ItemGroupList &armor_groups);
 	const ItemGroupList &getArmorGroups();
 	void setAnimation(v2f frame_range, float frame_speed, float frame_blend, bool frame_loop);
 	void getAnimation(v2f *frame_range, float *frame_speed, float *frame_blend, bool *frame_loop);
+	void setAnimationSpeed(float frame_speed);
 	void setBonePosition(const std::string &bone, v3f position, v3f rotation);
 	void getBonePosition(const std::string &bone, v3f *position, v3f *rotation);
 	void setAttachment(int parent_id, const std::string &bone, v3f position, v3f rotation);
 	void getAttachment(int *parent_id, std::string *bone, v3f *position, v3f *rotation);
+	void clearChildAttachments();
+	void clearParentAttachment();
 	void addAttachmentChild(int child_id);
 	void removeAttachmentChild(int child_id);
-	const UNORDERED_SET<int> &getAttachmentChildIds();
+	const std::unordered_set<int> &getAttachmentChildIds();
+	ServerActiveObject *getParent() const;
 	ObjectProperties* accessObjectProperties();
 	void notifyObjectPropertiesModified();
 protected:
-	s16 m_hp;
-	float m_yaw;
+	u16 m_hp = 1;
 
-	bool m_properties_sent;
-	struct ObjectProperties m_prop;
+	v3f m_rotation;
+
+	bool m_properties_sent = true;
+	ObjectProperties m_prop;
 
 	ItemGroupList m_armor_groups;
-	bool m_armor_groups_sent;
+	bool m_armor_groups_sent = false;
 
 	v2f m_animation_range;
-	float m_animation_speed;
-	float m_animation_blend;
-	bool m_animation_loop;
-	bool m_animation_sent;
+	float m_animation_speed = 0.0f;
+	float m_animation_blend = 0.0f;
+	bool m_animation_loop = true;
+	bool m_animation_sent = false;
+	bool m_animation_speed_sent = false;
 
 	// Stores position and rotation for each bone name
-	UNORDERED_MAP<std::string, core::vector2d<v3f> > m_bone_position;
-	bool m_bone_position_sent;
+	std::unordered_map<std::string, core::vector2d<v3f>> m_bone_position;
+	bool m_bone_position_sent = false;
 
-	int m_attachment_parent_id;
-	UNORDERED_SET<int> m_attachment_child_ids;
-	std::string m_attachment_bone;
+	int m_attachment_parent_id = 0;
+	std::unordered_set<int> m_attachment_child_ids;
+	std::string m_attachment_bone = "";
 	v3f m_attachment_position;
 	v3f m_attachment_rotation;
-	bool m_attachment_sent;
+	bool m_attachment_sent = false;
+private:
+	void onAttach(int parent_id);
+	void onDetach(int parent_id);
 };
 
 /*
@@ -102,6 +115,8 @@ public:
 			const std::string &data);
 	void step(float dtime, bool send_recommended);
 	std::string getClientInitializationData(u16 protocol_version);
+	bool isStaticAllowed() const
+	{ return m_prop.static_save; }
 	void getStaticData(std::string *result) const;
 	int punch(v3f dir,
 			const ToolCapabilities *toolcap=NULL,
@@ -112,10 +127,14 @@ public:
 	void moveTo(v3f pos, bool continuous);
 	float getMinimumSavedMovement();
 	std::string getDescription();
-	void setHP(s16 hp);
-	s16 getHP() const;
+	void setHP(s32 hp, const PlayerHPChangeReason &reason);
+	u16 getHP() const;
 	/* LuaEntitySAO-specific */
 	void setVelocity(v3f velocity);
+	void addVelocity(v3f velocity)
+	{
+		m_velocity += velocity;
+	}
 	v3f getVelocity();
 	void setAcceleration(v3f acceleration);
 	v3f getAcceleration();
@@ -126,6 +145,7 @@ public:
 			bool select_horiz_by_yawpitch);
 	std::string getName();
 	bool getCollisionBox(aabb3f *toset) const;
+	bool getSelectionBox(aabb3f *toset) const;
 	bool collideWithObjects() const;
 private:
 	std::string getPropertyPacket();
@@ -133,17 +153,17 @@ private:
 
 	std::string m_init_name;
 	std::string m_init_state;
-	bool m_registered;
+	bool m_registered = false;
 
 	v3f m_velocity;
 	v3f m_acceleration;
 
-	float m_last_sent_yaw;
 	v3f m_last_sent_position;
 	v3f m_last_sent_velocity;
-	float m_last_sent_position_timer;
-	float m_last_sent_move_precision;
-	std::string m_current_texture_modifier;
+	v3f m_last_sent_rotation;
+	float m_last_sent_position_timer = 0.0f;
+	float m_last_sent_move_precision = 0.0f;
+	std::string m_current_texture_modifier = "";
 };
 
 /*
@@ -152,11 +172,10 @@ private:
 
 class LagPool
 {
-	float m_pool;
-	float m_max;
+	float m_pool = 15.0f;
+	float m_max = 15.0f;
 public:
-	LagPool(): m_pool(15), m_max(15)
-	{}
+	LagPool() = default;
 
 	void setMax(float new_max)
 	{
@@ -188,13 +207,13 @@ public:
 	}
 };
 
-typedef UNORDERED_MAP<std::string, std::string> PlayerAttributes;
 class RemotePlayer;
 
 class PlayerSAO : public UnitSAO
 {
 public:
-	PlayerSAO(ServerEnvironment *env_, RemotePlayer *player_, u16 peer_id_, bool is_singleplayer);
+	PlayerSAO(ServerEnvironment *env_, RemotePlayer *player_, session_t peer_id_,
+			bool is_singleplayer);
 	~PlayerSAO();
 	ActiveObjectType getType() const
 	{ return ACTIVEOBJECT_TYPE_PLAYER; }
@@ -215,16 +234,16 @@ public:
 	void setBasePosition(const v3f &position);
 	void setPos(const v3f &pos);
 	void moveTo(v3f pos, bool continuous);
-	void setYaw(const float yaw);
+	void setPlayerYaw(const float yaw);
 	// Data should not be sent at player initialization
-	void setYawAndSend(const float yaw);
-	void setPitch(const float pitch);
+	void setPlayerYawAndSend(const float yaw);
+	void setLookPitch(const float pitch);
 	// Data should not be sent at player initialization
-	void setPitchAndSend(const float pitch);
-	f32 getPitch() const { return m_pitch; }
-	f32 getRadPitch() const { return m_pitch * core::DEGTORAD; }
+	void setLookPitchAndSend(const float pitch);
+	f32 getLookPitch() const { return m_pitch; }
+	f32 getRadLookPitch() const { return m_pitch * core::DEGTORAD; }
 	// Deprecated
-	f32 getRadPitchDep() const { return -1.0 * m_pitch * core::DEGTORAD; }
+	f32 getRadLookPitchDep() const { return -1.0 * m_pitch * core::DEGTORAD; }
 	void setFov(const float pitch);
 	f32 getFov() const { return m_fov; }
 	void setWantedRange(const s16 range);
@@ -239,8 +258,8 @@ public:
 		ServerActiveObject *puncher,
 		float time_from_last_punch);
 	void rightClick(ServerActiveObject *clicker) {}
-	void setHP(s16 hp);
-	void setHPRaw(s16 hp) { m_hp = hp; }
+	void setHP(s32 hp, const PlayerHPChangeReason &reason);
+	void setHPRaw(u16 hp) { m_hp = hp; }
 	s16 readDamage();
 	u16 getBreath() const { return m_breath; }
 	void setBreath(const u16 breath, bool send = true);
@@ -259,49 +278,6 @@ public:
 	int getWieldIndex() const;
 	void setWieldIndex(int i);
 
-	/*
-		Modding interface
-	*/
-	inline void setExtendedAttribute(const std::string &attr, const std::string &value)
-	{
-		m_extra_attributes[attr] = value;
-		m_extended_attributes_modified = true;
-	}
-
-	inline bool getExtendedAttribute(const std::string &attr, std::string *value)
-	{
-		if (m_extra_attributes.find(attr) == m_extra_attributes.end())
-			return false;
-
-		*value = m_extra_attributes[attr];
-		return true;
-	}
-
-	inline void removeExtendedAttribute(const std::string &attr)
-	{
-		PlayerAttributes::iterator it = m_extra_attributes.find(attr);
-		if (it == m_extra_attributes.end())
-			return;
-
-		m_extra_attributes.erase(it);
-		m_extended_attributes_modified = true;
-	}
-
-	inline const PlayerAttributes &getExtendedAttributes()
-	{
-		return m_extra_attributes;
-	}
-
-	inline bool extendedAttributesModified() const
-	{
-		return m_extended_attributes_modified;
-	}
-
-	inline void setExtendedAttributeModified(bool v)
-	{
-		m_extended_attributes_modified = v;
-	}
-
 	/*
 		PlayerSAO-specific
 	*/
@@ -309,7 +285,7 @@ public:
 	void disconnected();
 
 	RemotePlayer *getPlayer() { return m_player; }
-	u16 getPeerID() const { return m_peer_id; }
+	session_t getPeerID() const { return m_peer_id; }
 
 	// Cheat prevention
 
@@ -323,7 +299,7 @@ public:
 		m_time_from_last_punch = 0.0;
 		return r;
 	}
-	void noCheatDigStart(v3s16 p)
+	void noCheatDigStart(const v3s16 &p)
 	{
 		m_nocheat_dig_pos = p;
 		m_nocheat_dig_time = 0;
@@ -357,58 +333,124 @@ public:
 	}
 
 	bool getCollisionBox(aabb3f *toset) const;
+	bool getSelectionBox(aabb3f *toset) const;
 	bool collideWithObjects() const { return true; }
 
 	void finalize(RemotePlayer *player, const std::set<std::string> &privs);
 
 	v3f getEyePosition() const { return m_base_position + getEyeOffset(); }
 	v3f getEyeOffset() const;
+	float getZoomFOV() const;
+
+	inline Metadata &getMeta() { return m_meta; }
 
 private:
 	std::string getPropertyPacket();
 	void unlinkPlayerSessionAndSave();
 
-	RemotePlayer *m_player;
-	u16 m_peer_id;
-	Inventory *m_inventory;
-	s16 m_damage;
+	RemotePlayer *m_player = nullptr;
+	session_t m_peer_id = 0;
+	Inventory *m_inventory = nullptr;
 
 	// Cheat prevention
 	LagPool m_dig_pool;
 	LagPool m_move_pool;
 	v3f m_last_good_position;
-	float m_time_from_last_teleport;
-	float m_time_from_last_punch;
-	v3s16 m_nocheat_dig_pos;
-	float m_nocheat_dig_time;
+	float m_time_from_last_teleport = 0.0f;
+	float m_time_from_last_punch = 0.0f;
+	v3s16 m_nocheat_dig_pos = v3s16(32767, 32767, 32767);
+	float m_nocheat_dig_time = 0.0f;
 
 	// Timers
 	IntervalLimiter m_breathing_interval;
 	IntervalLimiter m_drowning_interval;
 	IntervalLimiter m_node_hurt_interval;
 
-	int m_wield_index;
-	bool m_position_not_sent;
+	int m_wield_index = 0;
+	bool m_position_not_sent = false;
 
 	// Cached privileges for enforcement
 	std::set<std::string> m_privs;
 	bool m_is_singleplayer;
 
-	u16 m_breath;
-	f32 m_pitch;
-	f32 m_fov;
-	s16 m_wanted_range;
+	u16 m_breath = PLAYER_MAX_BREATH_DEFAULT;
+	f32 m_pitch = 0.0f;
+	f32 m_fov = 0.0f;
+	s16 m_wanted_range = 0.0f;
 
-	PlayerAttributes m_extra_attributes;
-	bool m_extended_attributes_modified;
+	Metadata m_meta;
 public:
-	float m_physics_override_speed;
-	float m_physics_override_jump;
-	float m_physics_override_gravity;
-	bool m_physics_override_sneak;
-	bool m_physics_override_sneak_glitch;
-	bool m_physics_override_new_move;
-	bool m_physics_override_sent;
+	float m_physics_override_speed = 1.0f;
+	float m_physics_override_jump = 1.0f;
+	float m_physics_override_gravity = 1.0f;
+	bool m_physics_override_sneak = true;
+	bool m_physics_override_sneak_glitch = false;
+	bool m_physics_override_new_move = true;
+	bool m_physics_override_sent = false;
 };
 
-#endif
+
+struct PlayerHPChangeReason {
+	enum Type : u8 {
+		SET_HP,
+		PLAYER_PUNCH,
+		FALL,
+		NODE_DAMAGE,
+		DROWNING,
+		RESPAWN
+	};
+
+	Type type = SET_HP;
+	ServerActiveObject *object;
+	bool from_mod = false;
+	int lua_reference = -1;
+
+	inline bool hasLuaReference() const
+	{
+		return lua_reference >= 0;
+	}
+
+	bool setTypeFromString(const std::string &typestr)
+	{
+		if (typestr == "set_hp")
+			type = SET_HP;
+		else if (typestr == "punch")
+			type = PLAYER_PUNCH;
+		else if (typestr == "fall")
+			type = FALL;
+		else if (typestr == "node_damage")
+			type = NODE_DAMAGE;
+		else if (typestr == "drown")
+			type = DROWNING;
+		else if (typestr == "respawn")
+			type = RESPAWN;
+		else
+			return false;
+
+		return true;
+	}
+
+	std::string getTypeAsString() const
+	{
+		switch (type) {
+		case PlayerHPChangeReason::SET_HP:
+			return "set_hp";
+		case PlayerHPChangeReason::PLAYER_PUNCH:
+			return "punch";
+		case PlayerHPChangeReason::FALL:
+			return "fall";
+		case PlayerHPChangeReason::NODE_DAMAGE:
+			return "node_damage";
+		case PlayerHPChangeReason::DROWNING:
+			return "drown";
+		case PlayerHPChangeReason::RESPAWN:
+			return "respawn";
+		default:
+			return "?";
+		}
+	}
+
+	PlayerHPChangeReason(Type type, ServerActiveObject *object=NULL):
+			type(type), object(object)
+	{}
+};
