@@ -17,25 +17,26 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef EMERGE_HEADER
-#define EMERGE_HEADER
+#pragma once
 
 #include <map>
+#include <mutex>
+#include "network/networkprotocol.h"
 #include "irr_v3d.h"
 #include "util/container.h"
-#include "mapgen.h" // for MapgenParams
+#include "mapgen/mapgen.h" // for MapgenParams
 #include "map.h"
 
 #define BLOCK_EMERGE_ALLOW_GEN   (1 << 0)
 #define BLOCK_EMERGE_FORCE_QUEUE (1 << 1)
 
-#define EMERGE_DBG_OUT(x) do {                         \
+#define EMERGE_DBG_OUT(x) {                            \
 	if (enable_mapgen_debug_info)                      \
 		infostream << "EmergeThread: " x << std::endl; \
-} while (0)
+}
 
 class EmergeThread;
-class INodeDefManager;
+class NodeDefManager;
 class Settings;
 
 class BiomeManager;
@@ -46,19 +47,15 @@ class Server;
 
 // Structure containing inputs/outputs for chunk generation
 struct BlockMakeData {
-	MMVManip *vmanip;
-	u64 seed;
+	MMVManip *vmanip = nullptr;
+	u64 seed = 0;
 	v3s16 blockpos_min;
 	v3s16 blockpos_max;
 	v3s16 blockpos_requested;
 	UniqueQueue<v3s16> transforming_liquid;
-	INodeDefManager *nodedef;
+	const NodeDefManager *nodedef = nullptr;
 
-	BlockMakeData():
-		vmanip(NULL),
-		seed(0),
-		nodedef(NULL)
-	{}
+	BlockMakeData() = default;
 
 	~BlockMakeData() { delete vmanip; }
 };
@@ -91,11 +88,11 @@ struct BlockEmergeData {
 
 class EmergeManager {
 public:
-	INodeDefManager *ndef;
+	const NodeDefManager *ndef;
 	bool enable_mapgen_debug_info;
 
 	// Generation Notify
-	u32 gen_notify_on;
+	u32 gen_notify_on = 0;
 	std::set<u32> gen_notify_on_deco_ids;
 
 	// Parameters passed to mapgens owned by ServerMap
@@ -118,6 +115,7 @@ public:
 	// Methods
 	EmergeManager(Server *server);
 	~EmergeManager();
+	DISABLE_CLASS_COPY(EmergeManager);
 
 	bool initMapgens(MapgenParams *mgparams);
 
@@ -126,14 +124,14 @@ public:
 	bool isRunning();
 
 	bool enqueueBlockEmerge(
-		u16 peer_id,
+		session_t peer_id,
 		v3s16 blockpos,
 		bool allow_generate,
 		bool ignore_queue_limits=false);
 
 	bool enqueueBlockEmergeEx(
 		v3s16 blockpos,
-		u16 peer_id,
+		session_t peer_id,
 		u16 flags,
 		EmergeCompletionCallback callback,
 		void *callback_param);
@@ -143,7 +141,6 @@ public:
 	Mapgen *getCurrentMapgen();
 
 	// Mapgen helpers methods
-	Biome *getBiomeAtPoint(v3s16 p);
 	int getSpawnLevelAtPoint(v2s16 p);
 	int getGroundLevelAtPoint(v2s16 p);
 	bool isBlockUnderground(v3s16 blockpos);
@@ -153,11 +150,11 @@ public:
 private:
 	std::vector<Mapgen *> m_mapgens;
 	std::vector<EmergeThread *> m_threads;
-	bool m_threads_active;
+	bool m_threads_active = false;
 
-	Mutex m_queue_mutex;
+	std::mutex m_queue_mutex;
 	std::map<v3s16, BlockEmergeData> m_blocks_enqueued;
-	UNORDERED_MAP<u16, u16> m_peer_queue_count;
+	std::unordered_map<u16, u16> m_peer_queue_count;
 
 	u16 m_qlimit_total;
 	u16 m_qlimit_diskonly;
@@ -177,8 +174,4 @@ private:
 	bool popBlockEmergeData(v3s16 pos, BlockEmergeData *bedata);
 
 	friend class EmergeThread;
-
-	DISABLE_CLASS_COPY(EmergeManager);
 };
-
-#endif
