@@ -17,8 +17,7 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef MAP_HEADER
-#define MAP_HEADER
+#pragma once
 
 #include <iostream>
 #include <sstream>
@@ -32,9 +31,9 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 #include "voxel.h"
 #include "modifiedstate.h"
 #include "util/container.h"
-#include "util/cpp11_container.h"
 #include "nodetimer.h"
 #include "map_settings_manager.h"
+#include "debug.h"
 
 class Settings;
 class MapDatabase;
@@ -64,8 +63,7 @@ enum MapEditEventType{
 	MEET_REMOVENODE,
 	// Node swapped (changed without metadata change)
 	MEET_SWAPNODE,
-	// Node metadata of block changed (not knowing which node exactly)
-	// p stores block coordinate
+	// Node metadata changed
 	MEET_BLOCK_NODE_METADATA_CHANGED,
 	// Anything else (modified_blocks are set unsent)
 	MEET_OTHER
@@ -73,17 +71,13 @@ enum MapEditEventType{
 
 struct MapEditEvent
 {
-	MapEditEventType type;
+	MapEditEventType type = MEET_OTHER;
 	v3s16 p;
-	MapNode n;
+	MapNode n = CONTENT_AIR;
 	std::set<v3s16> modified_blocks;
-	u16 already_known_by_peer;
+	bool is_private_change = false;
 
-	MapEditEvent():
-		type(MEET_OTHER),
-		n(CONTENT_AIR),
-		already_known_by_peer(0)
-	{ }
+	MapEditEvent() = default;
 
 	MapEditEvent * clone()
 	{
@@ -92,6 +86,7 @@ struct MapEditEvent
 		event->p = p;
 		event->n = n;
 		event->modified_blocks = modified_blocks;
+		event->is_private_change = is_private_change;
 		return event;
 	}
 
@@ -113,11 +108,7 @@ struct MapEditEvent
 		case MEET_OTHER:
 		{
 			VoxelArea a;
-			for(std::set<v3s16>::iterator
-					i = modified_blocks.begin();
-					i != modified_blocks.end(); ++i)
-			{
-				v3s16 p = *i;
+			for (v3s16 p : modified_blocks) {
 				v3s16 np1 = p*MAP_BLOCKSIZE;
 				v3s16 np2 = np1 + v3s16(1,1,1)*MAP_BLOCKSIZE - v3s16(1,1,1);
 				a.addPoint(np1);
@@ -143,11 +134,7 @@ public:
 
 	Map(std::ostream &dout, IGameDef *gamedef);
 	virtual ~Map();
-
-	/*virtual u16 nodeContainerId() const
-	{
-		return NODECONTAINER_ID_MAP;
-	}*/
+	DISABLE_CLASS_COPY(Map);
 
 	virtual s32 mapType() const
 	{
@@ -181,8 +168,6 @@ public:
 		their differing fetch methods.
 	*/
 	virtual MapSector * emergeSector(v2s16 p){ return NULL; }
-	virtual MapSector * emergeSector(v2s16 p,
-			std::map<v3s16, MapBlock*> &changed_blocks){ return NULL; }
 
 	// Returns InvalidPositionException if not found
 	MapBlock * getBlockNoCreate(v3s16 p);
@@ -193,7 +178,7 @@ public:
 	virtual MapBlock * emergeBlock(v3s16 p, bool create_blank=true)
 	{ return getBlockNoCreateNoEx(p); }
 
-	inline INodeDefManager * getNodeDefManager() { return m_nodedef; }
+	inline const NodeDefManager * getNodeDefManager() { return m_nodedef; }
 
 	// Returns InvalidPositionException if not found
 	bool isNodeUnderground(v3s16 p);
@@ -225,18 +210,9 @@ public:
 	bool addNodeWithEvent(v3s16 p, MapNode n, bool remove_metadata = true);
 	bool removeNodeWithEvent(v3s16 p);
 
-	/*
-		Takes the blocks at the edges into account
-	*/
-	bool getDayNightDiff(v3s16 blockpos);
-
-	//core::aabbox3d<s16> getDisplayedBlockArea();
-
-	//bool updateChangedVisibleArea();
-
 	// Call these before and after saving of many blocks
-	virtual void beginSave() { return; }
-	virtual void endSave() { return; }
+	virtual void beginSave() {}
+	virtual void endSave() {}
 
 	virtual void save(ModifiedState save_level) { FATAL_ERROR("FIXME"); }
 
@@ -313,7 +289,6 @@ public:
 	*/
 
 	void transforming_liquid_add(v3s16 p);
-	s32 transforming_liquid_size();
 
 	bool isBlockOccluded(MapBlock *block, v3s16 cam_pos_nodes);
 protected:
@@ -328,25 +303,23 @@ protected:
 	std::map<v2s16, MapSector*> m_sectors;
 
 	// Be sure to set this to NULL when the cached sector is deleted
-	MapSector *m_sector_cache;
+	MapSector *m_sector_cache = nullptr;
 	v2s16 m_sector_cache_p;
 
 	// Queued transforming water nodes
 	UniqueQueue<v3s16> m_transforming_liquid;
 
 	// This stores the properties of the nodes on the map.
-	INodeDefManager *m_nodedef;
+	const NodeDefManager *m_nodedef;
 
 	bool isOccluded(v3s16 p0, v3s16 p1, float step, float stepfac,
 			float start_off, float end_off, u32 needed_count);
 
 private:
-	f32 m_transforming_liquid_loop_count_multiplier;
-	u32 m_unprocessed_count;
-	u64 m_inc_trending_up_start_time; // milliseconds
-	bool m_queue_size_timer_started;
-
-	DISABLE_CLASS_COPY(Map);
+	f32 m_transforming_liquid_loop_count_multiplier = 1.0f;
+	u32 m_unprocessed_count = 0;
+	u64 m_inc_trending_up_start_time = 0; // milliseconds
+	bool m_queue_size_timer_started = false;
 };
 
 /*
@@ -375,7 +348,7 @@ public:
 		- Check disk (doesn't load blocks)
 		- Create blank one
 	*/
-	ServerMapSector *createSector(v2s16 p);
+	MapSector *createSector(v2s16 p);
 
 	/*
 		Blocks are generated by using these and makeBlock().
@@ -442,18 +415,6 @@ public:
 
 	MapgenParams *getMapgenParams();
 
-	/*void saveChunkMeta();
-	void loadChunkMeta();*/
-
-	// The sector mutex should be locked when calling most of these
-
-	// This only saves sector-specific data such as the heightmap
-	// (no MapBlocks)
-	// DEPRECATED? Sectors have no metadata anymore.
-	void saveSectorMeta(ServerMapSector *sector);
-	MapSector* loadSectorMeta(std::string dirname, bool save_after_load);
-	bool loadSectorMeta(v2s16 p2d);
-
 	bool saveBlock(MapBlock *block);
 	static bool saveBlock(MapBlock *block, MapDatabase *db);
 	// This will generate a sector with getSector if not found.
@@ -506,8 +467,9 @@ private:
 		Metadata is re-written on disk only if this is true.
 		This is reset to false when written on disk.
 	*/
-	bool m_map_metadata_changed;
-	MapDatabase *dbase;
+	bool m_map_metadata_changed = true;
+	MapDatabase *dbase = nullptr;
+	MapDatabase *dbase_ro = nullptr;
 };
 
 
@@ -518,7 +480,7 @@ class MMVManip : public VoxelManipulator
 {
 public:
 	MMVManip(Map *map);
-	virtual ~MMVManip();
+	virtual ~MMVManip() = default;
 
 	virtual void clear()
 	{
@@ -526,9 +488,6 @@ public:
 		m_loaded_blocks.clear();
 	}
 
-	void setMap(Map *map)
-	{m_map = map;}
-
 	void initialEmerge(v3s16 blockpos_min, v3s16 blockpos_max,
 		bool load_if_inexistent = true);
 
@@ -536,10 +495,9 @@ public:
 	void blitBackAll(std::map<v3s16, MapBlock*> * modified_blocks,
 		bool overwrite_generated = true);
 
-	bool m_is_dirty;
+	bool m_is_dirty = false;
 
 protected:
-	bool m_create_area;
 	Map *m_map;
 	/*
 		key = blockpos
@@ -547,5 +505,3 @@ protected:
 	*/
 	std::map<v3s16, u8> m_loaded_blocks;
 };
-
-#endif
