@@ -17,23 +17,29 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#include "client.h"
+#include "client/client.h"
 
 #include "util/base64.h"
-#include "clientmedia.h"
+#include "chatmessage.h"
+#include "client/clientmedia.h"
 #include "log.h"
 #include "map.h"
 #include "mapsector.h"
-#include "minimap.h"
+#include "client/minimap.h"
+#include "modchannels.h"
 #include "nodedef.h"
 #include "serialization.h"
 #include "server.h"
 #include "util/strfnd.h"
+#include "client/clientevent.h"
+#include "client/sound.h"
 #include "network/clientopcodes.h"
+#include "network/connection.h"
 #include "script/scripting_client.h"
 #include "util/serialize.h"
 #include "util/srp.h"
 #include "tileanimation.h"
+#include "gettext.h"
 
 void Client::handleCommand_Deprecated(NetworkPacket* pkt)
 {
@@ -82,8 +88,8 @@ void Client::handleCommand_Hello(NetworkPacket* pkt)
 		// we recieved a TOCLIENT_HELLO while auth was already going on
 		errorstream << "Client: TOCLIENT_HELLO while auth was already going on"
 			<< "(chosen_mech=" << m_chosen_auth_mech << ")." << std::endl;
-		if ((m_chosen_auth_mech == AUTH_MECHANISM_SRP)
-				|| (m_chosen_auth_mech == AUTH_MECHANISM_LEGACY_PASSWORD)) {
+		if (m_chosen_auth_mech == AUTH_MECHANISM_SRP ||
+				m_chosen_auth_mech == AUTH_MECHANISM_LEGACY_PASSWORD) {
 			srp_user_delete((SRPUser *) m_auth_data);
 			m_auth_data = 0;
 		}
@@ -91,12 +97,19 @@ void Client::handleCommand_Hello(NetworkPacket* pkt)
 
 	// Authenticate using that method, or abort if there wasn't any method found
 	if (chosen_auth_mechanism != AUTH_MECHANISM_NONE) {
-		startAuth(chosen_auth_mechanism);
+		if (chosen_auth_mechanism == AUTH_MECHANISM_FIRST_SRP &&
+				!m_simple_singleplayer_mode &&
+				!getServerAddress().isLocalhost() &&
+				g_settings->getBool("enable_register_confirmation")) {
+			promptConfirmRegistration(chosen_auth_mechanism);
+		} else {
+			startAuth(chosen_auth_mechanism);
+		}
 	} else {
 		m_chosen_auth_mech = AUTH_MECHANISM_NONE;
 		m_access_denied = true;
 		m_access_denied_reason = "Unknown";
-		m_con.Disconnect();
+		m_con->Disconnect();
 	}
 
 }
@@ -121,7 +134,12 @@ void Client::handleCommand_AuthAccept(NetworkPacket* pkt)
 					<< m_recommended_send_interval<<std::endl;
 
 	// Reply to server
-	NetworkPacket resp_pkt(TOSERVER_INIT2, 0);
+	std::string lang = gettext("LANG_CODE");
+	if (lang == "LANG_CODE")
+		lang = "";
+
+	NetworkPacket resp_pkt(TOSERVER_INIT2, sizeof(u16) + lang.size());
+	resp_pkt << lang;
 	Send(&resp_pkt);
 
 	m_state = LC_Init;
@@ -142,64 +160,12 @@ void Client::handleCommand_AcceptSudoMode(NetworkPacket* pkt)
 }
 void Client::handleCommand_DenySudoMode(NetworkPacket* pkt)
 {
-	pushToChatQueue(L"Password change denied. Password NOT changed.");
+	ChatMessage *chatMessage = new ChatMessage(CHATMESSAGE_TYPE_SYSTEM,
+			L"Password change denied. Password NOT changed.");
+	pushToChatQueue(chatMessage);
 	// reset everything and be sad
 	deleteAuthData();
 }
-void Client::handleCommand_InitLegacy(NetworkPacket* pkt)
-{
-	if (pkt->getSize() < 1)
-		return;
-
-	u8 server_ser_ver;
-	*pkt >> server_ser_ver;
-
-	infostream << "Client: TOCLIENT_INIT_LEGACY received with "
-		"server_ser_ver=" << ((int)server_ser_ver & 0xff) << std::endl;
-
-	if (!ser_ver_supported(server_ser_ver)) {
-		infostream << "Client: TOCLIENT_INIT_LEGACY: Server sent "
-				<< "unsupported ser_fmt_ver"<< std::endl;
-		return;
-	}
-
-	m_server_ser_ver = server_ser_ver;
-
-	// We can be totally wrong with this guess
-	// but we only need some value < 25.
-	m_proto_ver = 24;
-
-	// Get player position
-	v3s16 playerpos_s16(0, BS * 2 + BS * 20, 0);
-	if (pkt->getSize() >= 1 + 6) {
-		*pkt >> playerpos_s16;
-	}
-	v3f playerpos_f = intToFloat(playerpos_s16, BS) - v3f(0, BS / 2, 0);
-
-
-	// Set player position
-	LocalPlayer *player = m_env.getLocalPlayer();
-	assert(player != NULL);
-	player->setPosition(playerpos_f);
-
-	if (pkt->getSize() >= 1 + 6 + 8) {
-		// Get map seed
-		*pkt >> m_map_seed;
-		infostream << "Client: received map seed: " << m_map_seed << std::endl;
-	}
-
-	if (pkt->getSize() >= 1 + 6 + 8 + 4) {
-		*pkt >> m_recommended_send_interval;
-		infostream << "Client: received recommended send interval "
-				<< m_recommended_send_interval<<std::endl;
-	}
-
-	// Reply to server
-	NetworkPacket resp_pkt(TOSERVER_INIT2, 0);
-	Send(&resp_pkt);
-
-	m_state = LC_Init;
-}
 
 void Client::handleCommand_AccessDenied(NetworkPacket* pkt)
 {
@@ -209,44 +175,44 @@ void Client::handleCommand_AccessDenied(NetworkPacket* pkt)
 	m_access_denied = true;
 	m_access_denied_reason = "Unknown";
 
-	if (pkt->getCommand() == TOCLIENT_ACCESS_DENIED) {
-		if (pkt->getSize() < 1)
-			return;
-
-		u8 denyCode = SERVER_ACCESSDENIED_UNEXPECTED_DATA;
-		*pkt >> denyCode;
-		if (denyCode == SERVER_ACCESSDENIED_SHUTDOWN ||
-				denyCode == SERVER_ACCESSDENIED_CRASH) {
-			*pkt >> m_access_denied_reason;
-			if (m_access_denied_reason == "") {
-				m_access_denied_reason = accessDeniedStrings[denyCode];
-			}
-			u8 reconnect;
-			*pkt >> reconnect;
-			m_access_denied_reconnect = reconnect & 1;
-		} else if (denyCode == SERVER_ACCESSDENIED_CUSTOM_STRING) {
-			*pkt >> m_access_denied_reason;
-		} else if (denyCode < SERVER_ACCESSDENIED_MAX) {
-			m_access_denied_reason = accessDeniedStrings[denyCode];
-		} else {
-			// Allow us to add new error messages to the
-			// protocol without raising the protocol version, if we want to.
-			// Until then (which may be never), this is outside
-			// of the defined protocol.
-			*pkt >> m_access_denied_reason;
-			if (m_access_denied_reason == "") {
-				m_access_denied_reason = "Unknown";
-			}
-		}
-	}
-	// 13/03/15 Legacy code from 0.4.12 and lesser. must stay 1 year
-	// for compat with old clients
-	else {
+	if (pkt->getCommand() != TOCLIENT_ACCESS_DENIED) {
+		// 13/03/15 Legacy code from 0.4.12 and lesser but is still used
+		// in some places of the server code
 		if (pkt->getSize() >= 2) {
 			std::wstring wide_reason;
 			*pkt >> wide_reason;
 			m_access_denied_reason = wide_to_utf8(wide_reason);
 		}
+		return;
+	}
+
+	if (pkt->getSize() < 1)
+		return;
+
+	u8 denyCode = SERVER_ACCESSDENIED_UNEXPECTED_DATA;
+	*pkt >> denyCode;
+	if (denyCode == SERVER_ACCESSDENIED_SHUTDOWN ||
+			denyCode == SERVER_ACCESSDENIED_CRASH) {
+		*pkt >> m_access_denied_reason;
+		if (m_access_denied_reason.empty()) {
+			m_access_denied_reason = accessDeniedStrings[denyCode];
+		}
+		u8 reconnect;
+		*pkt >> reconnect;
+		m_access_denied_reconnect = reconnect & 1;
+	} else if (denyCode == SERVER_ACCESSDENIED_CUSTOM_STRING) {
+		*pkt >> m_access_denied_reason;
+	} else if (denyCode < SERVER_ACCESSDENIED_MAX) {
+		m_access_denied_reason = accessDeniedStrings[denyCode];
+	} else {
+		// Allow us to add new error messages to the
+		// protocol without raising the protocol version, if we want to.
+		// Until then (which may be never), this is outside
+		// of the defined protocol.
+		*pkt >> m_access_denied_reason;
+		if (m_access_denied_reason.empty()) {
+			m_access_denied_reason = "Unknown";
+		}
 	}
 }
 
@@ -279,6 +245,33 @@ void Client::handleCommand_AddNode(NetworkPacket* pkt)
 
 	addNode(p, n, remove_metadata);
 }
+
+void Client::handleCommand_NodemetaChanged(NetworkPacket *pkt)
+{
+	if (pkt->getSize() < 1)
+		return;
+
+	std::istringstream is(pkt->readLongString(), std::ios::binary);
+	std::stringstream sstr;
+	decompressZlib(is, sstr);
+
+	NodeMetadataList meta_updates_list(false);
+	meta_updates_list.deSerialize(sstr, m_itemdef, true);
+
+	Map &map = m_env.getMap();
+	for (NodeMetadataMap::const_iterator i = meta_updates_list.begin();
+			i != meta_updates_list.end(); ++i) {
+		v3s16 pos = i->first;
+
+		if (map.isValidPosition(pos) &&
+				map.setNodeMetadata(pos, i->second))
+			continue; // Prevent from deleting metadata
+
+		// Meta couldn't be set, unused metadata
+		delete i->second;
+	}
+}
+
 void Client::handleCommand_BlockData(NetworkPacket* pkt)
 {
 	// Ignore too small packet
@@ -364,11 +357,11 @@ void Client::handleCommand_TimeOfDay(NetworkPacket* pkt)
 	}
 	else {
 		// Old message; try to approximate speed of time by ourselves
-		float time_of_day_f = (float)time_of_day / 24000.0;
+		float time_of_day_f = (float)time_of_day / 24000.0f;
 		float tod_diff_f = 0;
 
 		if (time_of_day_f < 0.2 && m_last_time_of_day_f > 0.8)
-			tod_diff_f = time_of_day_f - m_last_time_of_day_f + 1.0;
+			tod_diff_f = time_of_day_f - m_last_time_of_day_f + 1.0f;
 		else
 			tod_diff_f = time_of_day_f - m_last_time_of_day_f;
 
@@ -377,7 +370,7 @@ void Client::handleCommand_TimeOfDay(NetworkPacket* pkt)
 		m_time_of_day_update_timer = 0;
 
 		if (m_time_of_day_set) {
-			time_speed = (3600.0 * 24.0) * tod_diff_f / time_diff;
+			time_speed = (3600.0f * 24.0f) * tod_diff_f / time_diff;
 			infostream << "Client: Measured time_of_day speed (old format): "
 					<< time_speed << " tod_diff_f=" << tod_diff_f
 					<< " time_diff=" << time_diff << std::endl;
@@ -395,26 +388,39 @@ void Client::handleCommand_TimeOfDay(NetworkPacket* pkt)
 			<< " dr=" << dr << std::endl;
 }
 
-void Client::handleCommand_ChatMessage(NetworkPacket* pkt)
+void Client::handleCommand_ChatMessage(NetworkPacket *pkt)
 {
 	/*
-		u16 command
+		u8 version
+		u8 message_type
+		u16 sendername length
+		wstring sendername
 		u16 length
 		wstring message
-	*/
-	u16 len, read_wchar;
+	 */
 
-	*pkt >> len;
+	ChatMessage *chatMessage = new ChatMessage();
+	u8 version, message_type;
+	*pkt >> version >> message_type;
 
-	std::wstring message;
-	for (u32 i = 0; i < len; i++) {
-		*pkt >> read_wchar;
-		message += (wchar_t)read_wchar;
+	if (version != 1 || message_type >= CHATMESSAGE_TYPE_MAX) {
+		delete chatMessage;
+		return;
 	}
 
-	// If chat message not consummed by client lua API
-	if (!moddingEnabled() || !m_script->on_receiving_message(wide_to_utf8(message))) {
-		pushToChatQueue(message);
+	u64 timestamp;
+	*pkt >> chatMessage->sender >> chatMessage->message >> timestamp;
+	chatMessage->timestamp = static_cast<std::time_t>(timestamp);
+
+	chatMessage->type = (ChatMessageType) message_type;
+
+	// @TODO send this to CSM using ChatMessage object
+	if (moddingEnabled() && m_script->on_receiving_message(
+			wide_to_utf8(chatMessage->message))) {
+		// Message was consumed by CSM and should not be handled by client
+		delete chatMessage;
+	} else {
+		pushToChatQueue(chatMessage);
 	}
 }
 
@@ -519,9 +525,9 @@ void Client::handleCommand_HP(NetworkPacket* pkt)
 	LocalPlayer *player = m_env.getLocalPlayer();
 	assert(player != NULL);
 
-	u8 oldhp   = player->hp;
+	u16 oldhp   = player->hp;
 
-	u8 hp;
+	u16 hp;
 	*pkt >> hp;
 
 	player->hp = hp;
@@ -532,9 +538,9 @@ void Client::handleCommand_HP(NetworkPacket* pkt)
 
 	if (hp < oldhp) {
 		// Add to ClientEvent queue
-		ClientEvent event;
-		event.type = CE_PLAYER_DAMAGE;
-		event.player_damage.amount = oldhp - hp;
+		ClientEvent *event = new ClientEvent();
+		event->type = CE_PLAYER_DAMAGE;
+		event->player_damage.amount = oldhp - hp;
 		m_client_event_queue.push(event);
 	}
 }
@@ -575,10 +581,10 @@ void Client::handleCommand_MovePlayer(NetworkPacket* pkt)
 		it would just force the pitch and yaw values to whatever
 		the camera points to.
 	*/
-	ClientEvent event;
-	event.type = CE_PLAYER_FORCE_MOVE;
-	event.player_force_move.pitch = pitch;
-	event.player_force_move.yaw = yaw;
+	ClientEvent *event = new ClientEvent();
+	event->type = CE_PLAYER_FORCE_MOVE;
+	event->player_force_move.pitch = pitch;
+	event->player_force_move.yaw = yaw;
 	m_client_event_queue.push(event);
 }
 
@@ -590,12 +596,12 @@ void Client::handleCommand_DeathScreen(NetworkPacket* pkt)
 	*pkt >> set_camera_point_target;
 	*pkt >> camera_point_target;
 
-	ClientEvent event;
-	event.type                                = CE_DEATHSCREEN;
-	event.deathscreen.set_camera_point_target = set_camera_point_target;
-	event.deathscreen.camera_point_target_x   = camera_point_target.X;
-	event.deathscreen.camera_point_target_y   = camera_point_target.Y;
-	event.deathscreen.camera_point_target_z   = camera_point_target.Z;
+	ClientEvent *event = new ClientEvent();
+	event->type                                = CE_DEATHSCREEN;
+	event->deathscreen.set_camera_point_target = set_camera_point_target;
+	event->deathscreen.camera_point_target_x   = camera_point_target.X;
+	event->deathscreen.camera_point_target_y   = camera_point_target.Y;
+	event->deathscreen.camera_point_target_z   = camera_point_target.Z;
 	m_client_event_queue.push(event);
 }
 
@@ -641,7 +647,7 @@ void Client::handleCommand_AnnounceMedia(NetworkPacket* pkt)
 		Strfnd sf(str);
 		while(!sf.at_end()) {
 			std::string baseurl = trim(sf.next(","));
-			if (baseurl != "")
+			if (!baseurl.empty())
 				m_media_downloader->addRemoteServer(baseurl);
 		}
 	}
@@ -679,8 +685,7 @@ void Client::handleCommand_Media(NetworkPacket* pkt)
 	if (num_files == 0)
 		return;
 
-	if (m_media_downloader == NULL ||
-			!m_media_downloader->isStarted()) {
+	if (!m_media_downloader || !m_media_downloader->isStarted()) {
 		const char *problem = m_media_downloader ?
 			"media has not been requested" :
 			"all media has been received already";
@@ -760,6 +765,7 @@ void Client::handleCommand_PlaySound(NetworkPacket* pkt)
 		[23 + len] u16 object_id
 		[25 + len] bool loop
 		[26 + len] f32 fade
+		[30 + len] f32 pitch
 	*/
 
 	s32 server_id;
@@ -770,29 +776,31 @@ void Client::handleCommand_PlaySound(NetworkPacket* pkt)
 	v3f pos;
 	u16 object_id;
 	bool loop;
-	float fade = 0;
+	float fade = 0.0f;
+	float pitch = 1.0f;
 
 	*pkt >> server_id >> name >> gain >> type >> pos >> object_id >> loop;
 
 	try {
 		*pkt >> fade;
+		*pkt >> pitch;
 	} catch (PacketError &e) {};
 
 	// Start playing
 	int client_id = -1;
 	switch(type) {
 		case 0: // local
-			client_id = m_sound->playSound(name, loop, gain, fade);
+			client_id = m_sound->playSound(name, loop, gain, fade, pitch);
 			break;
 		case 1: // positional
-			client_id = m_sound->playSoundAt(name, loop, gain, pos);
+			client_id = m_sound->playSoundAt(name, loop, gain, pos, pitch);
 			break;
 		case 2:
 		{ // object
 			ClientActiveObject *cao = m_env.getActiveObject(object_id);
 			if (cao)
 				pos = cao->getPosition();
-			client_id = m_sound->playSoundAt(name, loop, gain, pos);
+			client_id = m_sound->playSoundAt(name, loop, gain, pos, pitch);
 			// TODO: Set up sound to move with object
 			break;
 		}
@@ -814,7 +822,7 @@ void Client::handleCommand_StopSound(NetworkPacket* pkt)
 
 	*pkt >> server_id;
 
-	UNORDERED_MAP<s32, int>::iterator i = m_sounds_server_to_client.find(server_id);
+	std::unordered_map<s32, int>::iterator i = m_sounds_server_to_client.find(server_id);
 	if (i != m_sounds_server_to_client.end()) {
 		int client_id = i->second;
 		m_sound->stopSound(client_id);
@@ -829,7 +837,7 @@ void Client::handleCommand_FadeSound(NetworkPacket *pkt)
 
 	*pkt >> sound_id >> step >> gain;
 
-	UNORDERED_MAP<s32, int>::iterator i =
+	std::unordered_map<s32, int>::const_iterator i =
 			m_sounds_server_to_client.find(sound_id);
 
 	if (i != m_sounds_server_to_client.end())
@@ -866,21 +874,34 @@ void Client::handleCommand_InventoryFormSpec(NetworkPacket* pkt)
 
 void Client::handleCommand_DetachedInventory(NetworkPacket* pkt)
 {
-	std::string datastring(pkt->getString(0), pkt->getSize());
-	std::istringstream is(datastring, std::ios_base::binary);
-
-	std::string name = deSerializeString(is);
+	std::string name;
+	bool keep_inv = true;
+	*pkt >> name >> keep_inv;
 
 	infostream << "Client: Detached inventory update: \"" << name
-			<< "\"" << std::endl;
+		<< "\", mode=" << (keep_inv ? "update" : "remove") << std::endl;
 
-	Inventory *inv = NULL;
-	if (m_detached_inventories.count(name) > 0)
-		inv = m_detached_inventories[name];
-	else {
+	const auto &inv_it = m_detached_inventories.find(name);
+	if (!keep_inv) {
+		if (inv_it != m_detached_inventories.end()) {
+			delete inv_it->second;
+			m_detached_inventories.erase(inv_it);
+		}
+		return;
+	}
+	Inventory *inv = nullptr;
+	if (inv_it == m_detached_inventories.end()) {
 		inv = new Inventory(m_itemdef);
 		m_detached_inventories[name] = inv;
+	} else {
+		inv = inv_it->second;
 	}
+
+	u16 ignore;
+	*pkt >> ignore; // this used to be the length of the following string, ignore it
+
+	std::string contents = pkt->getRemainingString();
+	std::istringstream is(contents, std::ios::binary);
 	inv->deSerialize(is);
 }
 
@@ -891,12 +912,12 @@ void Client::handleCommand_ShowFormSpec(NetworkPacket* pkt)
 
 	*pkt >> formname;
 
-	ClientEvent event;
-	event.type = CE_SHOW_FORMSPEC;
+	ClientEvent *event = new ClientEvent();
+	event->type = CE_SHOW_FORMSPEC;
 	// pointer is required as event is a struct only!
 	// adding a std:string to a struct isn't possible
-	event.show_formspec.formspec = new std::string(formspec);
-	event.show_formspec.formname = new std::string(formname);
+	event->show_formspec.formspec = new std::string(formspec);
+	event->show_formspec.formname = new std::string(formname);
 	m_client_event_queue.push(event);
 }
 
@@ -905,38 +926,42 @@ void Client::handleCommand_SpawnParticle(NetworkPacket* pkt)
 	std::string datastring(pkt->getString(0), pkt->getSize());
 	std::istringstream is(datastring, std::ios_base::binary);
 
-	v3f pos                 = readV3F1000(is);
-	v3f vel                 = readV3F1000(is);
-	v3f acc                 = readV3F1000(is);
-	float expirationtime    = readF1000(is);
-	float size              = readF1000(is);
+	v3f pos                 = readV3F32(is);
+	v3f vel                 = readV3F32(is);
+	v3f acc                 = readV3F32(is);
+	float expirationtime    = readF32(is);
+	float size              = readF32(is);
 	bool collisiondetection = readU8(is);
 	std::string texture     = deSerializeLongString(is);
-	bool vertical           = false;
-	bool collision_removal  = false;
-	struct TileAnimationParams animation;
-	animation.type = TAT_NONE;
-	u8 glow = 0;
+
+	bool vertical          = false;
+	bool collision_removal = false;
+	TileAnimationParams animation;
+	animation.type         = TAT_NONE;
+	u8 glow                = 0;
+	bool object_collision  = false;
 	try {
 		vertical = readU8(is);
 		collision_removal = readU8(is);
 		animation.deSerialize(is, m_proto_ver);
 		glow = readU8(is);
+		object_collision = readU8(is);
 	} catch (...) {}
 
-	ClientEvent event;
-	event.type                              = CE_SPAWN_PARTICLE;
-	event.spawn_particle.pos                = new v3f (pos);
-	event.spawn_particle.vel                = new v3f (vel);
-	event.spawn_particle.acc                = new v3f (acc);
-	event.spawn_particle.expirationtime     = expirationtime;
-	event.spawn_particle.size               = size;
-	event.spawn_particle.collisiondetection = collisiondetection;
-	event.spawn_particle.collision_removal  = collision_removal;
-	event.spawn_particle.vertical           = vertical;
-	event.spawn_particle.texture            = new std::string(texture);
-	event.spawn_particle.animation          = animation;
-	event.spawn_particle.glow               = glow;
+	ClientEvent *event = new ClientEvent();
+	event->type                              = CE_SPAWN_PARTICLE;
+	event->spawn_particle.pos                = new v3f (pos);
+	event->spawn_particle.vel                = new v3f (vel);
+	event->spawn_particle.acc                = new v3f (acc);
+	event->spawn_particle.expirationtime     = expirationtime;
+	event->spawn_particle.size               = size;
+	event->spawn_particle.collisiondetection = collisiondetection;
+	event->spawn_particle.collision_removal  = collision_removal;
+	event->spawn_particle.object_collision   = object_collision;
+	event->spawn_particle.vertical           = vertical;
+	event->spawn_particle.texture            = new std::string(texture);
+	event->spawn_particle.animation          = animation;
+	event->spawn_particle.glow               = glow;
 
 	m_client_event_queue.push(event);
 }
@@ -956,7 +981,7 @@ void Client::handleCommand_AddParticleSpawner(NetworkPacket* pkt)
 	float minsize;
 	float maxsize;
 	bool collisiondetection;
-	u32 id;
+	u32 server_id;
 
 	*pkt >> amount >> spawntime >> minpos >> maxpos >> minvel >> maxvel
 		>> minacc >> maxacc >> minexptime >> maxexptime >> minsize
@@ -964,14 +989,15 @@ void Client::handleCommand_AddParticleSpawner(NetworkPacket* pkt)
 
 	std::string texture = pkt->readLongString();
 
-	*pkt >> id;
+	*pkt >> server_id;
 
-	bool vertical = false;
+	bool vertical          = false;
 	bool collision_removal = false;
-	struct TileAnimationParams animation;
-	animation.type = TAT_NONE;
-	u8 glow = 0;
-	u16 attached_id = 0;
+	u16 attached_id        = 0;
+	TileAnimationParams animation;
+	animation.type         = TAT_NONE;
+	u8 glow                = 0;
+	bool object_collision  = false;
 	try {
 		*pkt >> vertical;
 		*pkt >> collision_removal;
@@ -982,30 +1008,32 @@ void Client::handleCommand_AddParticleSpawner(NetworkPacket* pkt)
 		std::istringstream is(datastring, std::ios_base::binary);
 		animation.deSerialize(is, m_proto_ver);
 		glow = readU8(is);
+		object_collision = readU8(is);
 	} catch (...) {}
 
-	ClientEvent event;
-	event.type                                   = CE_ADD_PARTICLESPAWNER;
-	event.add_particlespawner.amount             = amount;
-	event.add_particlespawner.spawntime          = spawntime;
-	event.add_particlespawner.minpos             = new v3f (minpos);
-	event.add_particlespawner.maxpos             = new v3f (maxpos);
-	event.add_particlespawner.minvel             = new v3f (minvel);
-	event.add_particlespawner.maxvel             = new v3f (maxvel);
-	event.add_particlespawner.minacc             = new v3f (minacc);
-	event.add_particlespawner.maxacc             = new v3f (maxacc);
-	event.add_particlespawner.minexptime         = minexptime;
-	event.add_particlespawner.maxexptime         = maxexptime;
-	event.add_particlespawner.minsize            = minsize;
-	event.add_particlespawner.maxsize            = maxsize;
-	event.add_particlespawner.collisiondetection = collisiondetection;
-	event.add_particlespawner.collision_removal  = collision_removal;
-	event.add_particlespawner.attached_id        = attached_id;
-	event.add_particlespawner.vertical           = vertical;
-	event.add_particlespawner.texture            = new std::string(texture);
-	event.add_particlespawner.id                 = id;
-	event.add_particlespawner.animation          = animation;
-	event.add_particlespawner.glow               = glow;
+	auto event = new ClientEvent();
+	event->type                                   = CE_ADD_PARTICLESPAWNER;
+	event->add_particlespawner.amount             = amount;
+	event->add_particlespawner.spawntime          = spawntime;
+	event->add_particlespawner.minpos             = new v3f (minpos);
+	event->add_particlespawner.maxpos             = new v3f (maxpos);
+	event->add_particlespawner.minvel             = new v3f (minvel);
+	event->add_particlespawner.maxvel             = new v3f (maxvel);
+	event->add_particlespawner.minacc             = new v3f (minacc);
+	event->add_particlespawner.maxacc             = new v3f (maxacc);
+	event->add_particlespawner.minexptime         = minexptime;
+	event->add_particlespawner.maxexptime         = maxexptime;
+	event->add_particlespawner.minsize            = minsize;
+	event->add_particlespawner.maxsize            = maxsize;
+	event->add_particlespawner.collisiondetection = collisiondetection;
+	event->add_particlespawner.collision_removal  = collision_removal;
+	event->add_particlespawner.object_collision   = object_collision;
+	event->add_particlespawner.attached_id        = attached_id;
+	event->add_particlespawner.vertical           = vertical;
+	event->add_particlespawner.texture            = new std::string(texture);
+	event->add_particlespawner.id                 = server_id;
+	event->add_particlespawner.animation          = animation;
+	event->add_particlespawner.glow               = glow;
 
 	m_client_event_queue.push(event);
 }
@@ -1013,22 +1041,12 @@ void Client::handleCommand_AddParticleSpawner(NetworkPacket* pkt)
 
 void Client::handleCommand_DeleteParticleSpawner(NetworkPacket* pkt)
 {
-	u16 legacy_id;
-	u32 id;
+	u32 server_id;
+	*pkt >> server_id;
 
-	// Modification set 13/03/15, 1 year of compat for protocol v24
-	if (pkt->getCommand() == TOCLIENT_DELETE_PARTICLESPAWNER_LEGACY) {
-		*pkt >> legacy_id;
-	}
-	else {
-		*pkt >> id;
-	}
-
-
-	ClientEvent event;
-	event.type                      = CE_DELETE_PARTICLESPAWNER;
-	event.delete_particlespawner.id =
-			(pkt->getCommand() == TOCLIENT_DELETE_PARTICLESPAWNER_LEGACY ? (u32) legacy_id : id);
+	ClientEvent *event = new ClientEvent();
+	event->type = CE_DELETE_PARTICLESPAWNER;
+	event->delete_particlespawner.id = server_id;
 
 	m_client_event_queue.push(event);
 }
@@ -1038,7 +1056,7 @@ void Client::handleCommand_HudAdd(NetworkPacket* pkt)
 	std::string datastring(pkt->getString(0), pkt->getSize());
 	std::istringstream is(datastring, std::ios_base::binary);
 
-	u32 id;
+	u32 server_id;
 	u8 type;
 	v2f pos;
 	std::string name;
@@ -1052,7 +1070,7 @@ void Client::handleCommand_HudAdd(NetworkPacket* pkt)
 	v3f world_pos;
 	v2s32 size;
 
-	*pkt >> id >> type >> pos >> name >> scale >> text >> number >> item
+	*pkt >> server_id >> type >> pos >> name >> scale >> text >> number >> item
 		>> dir >> align >> offset;
 	try {
 		*pkt >> world_pos;
@@ -1063,34 +1081,40 @@ void Client::handleCommand_HudAdd(NetworkPacket* pkt)
 		*pkt >> size;
 	} catch(SerializationError &e) {};
 
-	ClientEvent event;
-	event.type             = CE_HUDADD;
-	event.hudadd.id        = id;
-	event.hudadd.type      = type;
-	event.hudadd.pos       = new v2f(pos);
-	event.hudadd.name      = new std::string(name);
-	event.hudadd.scale     = new v2f(scale);
-	event.hudadd.text      = new std::string(text);
-	event.hudadd.number    = number;
-	event.hudadd.item      = item;
-	event.hudadd.dir       = dir;
-	event.hudadd.align     = new v2f(align);
-	event.hudadd.offset    = new v2f(offset);
-	event.hudadd.world_pos = new v3f(world_pos);
-	event.hudadd.size      = new v2s32(size);
+	ClientEvent *event = new ClientEvent();
+	event->type             = CE_HUDADD;
+	event->hudadd.server_id = server_id;
+	event->hudadd.type      = type;
+	event->hudadd.pos       = new v2f(pos);
+	event->hudadd.name      = new std::string(name);
+	event->hudadd.scale     = new v2f(scale);
+	event->hudadd.text      = new std::string(text);
+	event->hudadd.number    = number;
+	event->hudadd.item      = item;
+	event->hudadd.dir       = dir;
+	event->hudadd.align     = new v2f(align);
+	event->hudadd.offset    = new v2f(offset);
+	event->hudadd.world_pos = new v3f(world_pos);
+	event->hudadd.size      = new v2s32(size);
 	m_client_event_queue.push(event);
 }
 
 void Client::handleCommand_HudRemove(NetworkPacket* pkt)
 {
-	u32 id;
+	u32 server_id;
 
-	*pkt >> id;
+	*pkt >> server_id;
 
-	ClientEvent event;
-	event.type     = CE_HUDRM;
-	event.hudrm.id = id;
-	m_client_event_queue.push(event);
+	auto i = m_hud_server_to_client.find(server_id);
+	if (i != m_hud_server_to_client.end()) {
+		int client_id = i->second;
+		m_hud_server_to_client.erase(i);
+
+		ClientEvent *event = new ClientEvent();
+		event->type     = CE_HUDRM;
+		event->hudrm.id = client_id;
+		m_client_event_queue.push(event);
+	}
 }
 
 void Client::handleCommand_HudChange(NetworkPacket* pkt)
@@ -1100,10 +1124,10 @@ void Client::handleCommand_HudChange(NetworkPacket* pkt)
 	v3f v3fdata;
 	u32 intdata = 0;
 	v2s32 v2s32data;
-	u32 id;
+	u32 server_id;
 	u8 stat;
 
-	*pkt >> id >> stat;
+	*pkt >> server_id >> stat;
 
 	if (stat == HUD_STAT_POS || stat == HUD_STAT_SCALE ||
 		stat == HUD_STAT_ALIGN || stat == HUD_STAT_OFFSET)
@@ -1117,16 +1141,19 @@ void Client::handleCommand_HudChange(NetworkPacket* pkt)
 	else
 		*pkt >> intdata;
 
-	ClientEvent event;
-	event.type              = CE_HUDCHANGE;
-	event.hudchange.id      = id;
-	event.hudchange.stat    = (HudElementStat)stat;
-	event.hudchange.v2fdata = new v2f(v2fdata);
-	event.hudchange.v3fdata = new v3f(v3fdata);
-	event.hudchange.sdata   = new std::string(sdata);
-	event.hudchange.data    = intdata;
-	event.hudchange.v2s32data = new v2s32(v2s32data);
-	m_client_event_queue.push(event);
+	std::unordered_map<u32, u32>::const_iterator i = m_hud_server_to_client.find(server_id);
+	if (i != m_hud_server_to_client.end()) {
+		ClientEvent *event = new ClientEvent();
+		event->type              = CE_HUDCHANGE;
+		event->hudchange.id      = i->second;
+		event->hudchange.stat    = (HudElementStat)stat;
+		event->hudchange.v2fdata = new v2f(v2fdata);
+		event->hudchange.v3fdata = new v3f(v3fdata);
+		event->hudchange.sdata   = new std::string(sdata);
+		event->hudchange.data    = intdata;
+		event->hudchange.v2s32data = new v2s32(v2s32data);
+		m_client_event_queue.push(event);
+	}
 }
 
 void Client::handleCommand_HudSetFlags(NetworkPacket* pkt)
@@ -1139,18 +1166,23 @@ void Client::handleCommand_HudSetFlags(NetworkPacket* pkt)
 	assert(player != NULL);
 
 	bool was_minimap_visible = player->hud_flags & HUD_FLAG_MINIMAP_VISIBLE;
+	bool was_minimap_radar_visible = player->hud_flags & HUD_FLAG_MINIMAP_RADAR_VISIBLE;
 
 	player->hud_flags &= ~mask;
 	player->hud_flags |= flags;
 
 	m_minimap_disabled_by_server = !(player->hud_flags & HUD_FLAG_MINIMAP_VISIBLE);
+	bool m_minimap_radar_disabled_by_server = !(player->hud_flags & HUD_FLAG_MINIMAP_RADAR_VISIBLE);
 
 	// Hide minimap if it has been disabled by the server
-	if (m_minimap && m_minimap_disabled_by_server && was_minimap_visible) {
+	if (m_minimap && m_minimap_disabled_by_server && was_minimap_visible)
 		// defers a minimap update, therefore only call it if really
 		// needed, by checking that minimap was visible before
 		m_minimap->setMinimapMode(MINIMAP_MODE_OFF);
-	}
+
+	// Switch to surface mode if radar disabled by server
+	if (m_minimap && m_minimap_radar_disabled_by_server && was_minimap_radar_visible)
+		m_minimap->setMinimapMode(MINIMAP_MODE_SURFACEx1);
 }
 
 void Client::handleCommand_HudSetParam(NetworkPacket* pkt)
@@ -1169,7 +1201,7 @@ void Client::handleCommand_HudSetParam(NetworkPacket* pkt)
 	}
 	else if (param == HUD_PARAM_HOTBAR_IMAGE) {
 		// If value not empty verify image exists in texture source
-		if (value != "" && !getTextureSource()->isKnownSourceImage(value)) {
+		if (!value.empty() && !getTextureSource()->isKnownSourceImage(value)) {
 			errorstream << "Server sent wrong Hud hotbar image (sent value: '"
 				<< value << "')" << std::endl;
 			return;
@@ -1178,7 +1210,7 @@ void Client::handleCommand_HudSetParam(NetworkPacket* pkt)
 	}
 	else if (param == HUD_PARAM_HOTBAR_SELECTED_IMAGE) {
 		// If value not empty verify image exists in texture source
-		if (value != "" && !getTextureSource()->isKnownSourceImage(value)) {
+		if (!value.empty() && !getTextureSource()->isKnownSourceImage(value)) {
 			errorstream << "Server sent wrong Hud hotbar selected image (sent value: '"
 					<< value << "')" << std::endl;
 			return;
@@ -1205,12 +1237,12 @@ void Client::handleCommand_HudSetSky(NetworkPacket* pkt)
 		clouds = readU8(is);
 	} catch (...) {}
 
-	ClientEvent event;
-	event.type            = CE_SET_SKY;
-	event.set_sky.bgcolor = bgcolor;
-	event.set_sky.type    = type;
-	event.set_sky.params  = params;
-	event.set_sky.clouds  = clouds;
+	ClientEvent *event = new ClientEvent();
+	event->type            = CE_SET_SKY;
+	event->set_sky.bgcolor = bgcolor;
+	event->set_sky.type    = type;
+	event->set_sky.params  = params;
+	event->set_sky.clouds  = clouds;
 	m_client_event_queue.push(event);
 }
 
@@ -1226,19 +1258,19 @@ void Client::handleCommand_CloudParams(NetworkPacket* pkt)
 	*pkt >> density >> color_bright >> color_ambient
 			>> height >> thickness >> speed;
 
-	ClientEvent event;
-	event.type                       = CE_CLOUD_PARAMS;
-	event.cloud_params.density       = density;
+	ClientEvent *event = new ClientEvent();
+	event->type                       = CE_CLOUD_PARAMS;
+	event->cloud_params.density       = density;
 	// use the underlying u32 representation, because we can't
 	// use struct members with constructors here, and this way
 	// we avoid using new() and delete() for no good reason
-	event.cloud_params.color_bright  = color_bright.color;
-	event.cloud_params.color_ambient = color_ambient.color;
-	event.cloud_params.height        = height;
-	event.cloud_params.thickness     = thickness;
+	event->cloud_params.color_bright  = color_bright.color;
+	event->cloud_params.color_ambient = color_ambient.color;
+	event->cloud_params.height        = height;
+	event->cloud_params.thickness     = thickness;
 	// same here: deconstruct to skip constructor
-	event.cloud_params.speed_x       = speed.X;
-	event.cloud_params.speed_y       = speed.Y;
+	event->cloud_params.speed_x       = speed.X;
+	event->cloud_params.speed_y       = speed.Y;
 	m_client_event_queue.push(event);
 }
 
@@ -1251,10 +1283,10 @@ void Client::handleCommand_OverrideDayNightRatio(NetworkPacket* pkt)
 
 	float day_night_ratio_f = (float)day_night_ratio_u / 65536;
 
-	ClientEvent event;
-	event.type                                 = CE_OVERRIDE_DAY_NIGHT_RATIO;
-	event.override_day_night_ratio.do_override = do_override;
-	event.override_day_night_ratio.ratio_f     = day_night_ratio_f;
+	ClientEvent *event = new ClientEvent();
+	event->type                                 = CE_OVERRIDE_DAY_NIGHT_RATIO;
+	event->override_day_night_ratio.do_override = do_override;
+	event->override_day_night_ratio.ratio_f     = day_night_ratio_f;
 	m_client_event_queue.push(event);
 }
 
@@ -1278,11 +1310,33 @@ void Client::handleCommand_EyeOffset(NetworkPacket* pkt)
 	*pkt >> player->eye_offset_first >> player->eye_offset_third;
 }
 
+void Client::handleCommand_UpdatePlayerList(NetworkPacket* pkt)
+{
+	u8 type;
+	u16 num_players;
+	*pkt >> type >> num_players;
+	PlayerListModifer notice_type = (PlayerListModifer) type;
+
+	for (u16 i = 0; i < num_players; i++) {
+		std::string name;
+		*pkt >> name;
+		switch (notice_type) {
+		case PLAYER_LIST_INIT:
+		case PLAYER_LIST_ADD:
+			m_env.addPlayerName(name);
+			continue;
+		case PLAYER_LIST_REMOVE:
+			m_env.removePlayerName(name);
+			continue;
+		}
+	}
+}
+
 void Client::handleCommand_SrpBytesSandB(NetworkPacket* pkt)
 {
-	if ((m_chosen_auth_mech != AUTH_MECHANISM_LEGACY_PASSWORD)
-			&& (m_chosen_auth_mech != AUTH_MECHANISM_SRP)) {
-		errorstream << "Client: Recieved SRP S_B login message,"
+	if (m_chosen_auth_mech != AUTH_MECHANISM_SRP &&
+			m_chosen_auth_mech != AUTH_MECHANISM_LEGACY_PASSWORD) {
+		errorstream << "Client: Received SRP S_B login message,"
 			<< " but wasn't supposed to (chosen_mech="
 			<< m_chosen_auth_mech << ")." << std::endl;
 		return;
@@ -1295,7 +1349,7 @@ void Client::handleCommand_SrpBytesSandB(NetworkPacket* pkt)
 	std::string B;
 	*pkt >> s >> B;
 
-	infostream << "Client: Recieved TOCLIENT_SRP_BYTES_S_B." << std::endl;
+	infostream << "Client: Received TOCLIENT_SRP_BYTES_S_B." << std::endl;
 
 	srp_user_process_challenge(usr, (const unsigned char *) s.c_str(), s.size(),
 		(const unsigned char *) B.c_str(), B.size(),
@@ -1310,3 +1364,113 @@ void Client::handleCommand_SrpBytesSandB(NetworkPacket* pkt)
 	resp_pkt << std::string(bytes_M, len_M);
 	Send(&resp_pkt);
 }
+
+void Client::handleCommand_FormspecPrepend(NetworkPacket *pkt)
+{
+	LocalPlayer *player = m_env.getLocalPlayer();
+	assert(player != NULL);
+
+	// Store formspec in LocalPlayer
+	*pkt >> player->formspec_prepend;
+}
+
+void Client::handleCommand_CSMRestrictionFlags(NetworkPacket *pkt)
+{
+	*pkt >> m_csm_restriction_flags >> m_csm_restriction_noderange;
+
+	// Restrictions were received -> load mods if it's enabled
+	// Note: this should be moved after mods receptions from server instead
+	loadMods();
+}
+
+/*
+ * Mod channels
+ */
+
+void Client::handleCommand_ModChannelMsg(NetworkPacket *pkt)
+{
+	std::string channel_name, sender, channel_msg;
+	*pkt >> channel_name >> sender >> channel_msg;
+
+	verbosestream << "Mod channel message received from server " << pkt->getPeerId()
+		<< " on channel " << channel_name << ". sender: `" << sender << "`, message: "
+		<< channel_msg << std::endl;
+
+	if (!m_modchannel_mgr->channelRegistered(channel_name)) {
+		verbosestream << "Server sent us messages on unregistered channel "
+			<< channel_name << ", ignoring." << std::endl;
+		return;
+	}
+
+	m_script->on_modchannel_message(channel_name, sender, channel_msg);
+}
+
+void Client::handleCommand_ModChannelSignal(NetworkPacket *pkt)
+{
+	u8 signal_tmp;
+	ModChannelSignal signal;
+	std::string channel;
+
+	*pkt >> signal_tmp >> channel;
+
+	signal = (ModChannelSignal)signal_tmp;
+
+	bool valid_signal = true;
+	// @TODO: send Signal to Lua API
+	switch (signal) {
+		case MODCHANNEL_SIGNAL_JOIN_OK:
+			m_modchannel_mgr->setChannelState(channel, MODCHANNEL_STATE_READ_WRITE);
+			infostream << "Server ack our mod channel join on channel `" << channel
+				<< "`, joining." << std::endl;
+			break;
+		case MODCHANNEL_SIGNAL_JOIN_FAILURE:
+			// Unable to join, remove channel
+			m_modchannel_mgr->leaveChannel(channel, 0);
+			infostream << "Server refused our mod channel join on channel `" << channel
+				<< "`" << std::endl;
+			break;
+		case MODCHANNEL_SIGNAL_LEAVE_OK:
+#ifndef NDEBUG
+			infostream << "Server ack our mod channel leave on channel " << channel
+				<< "`, leaving." << std::endl;
+#endif
+			break;
+		case MODCHANNEL_SIGNAL_LEAVE_FAILURE:
+			infostream << "Server refused our mod channel leave on channel `" << channel
+				<< "`" << std::endl;
+			break;
+		case MODCHANNEL_SIGNAL_CHANNEL_NOT_REGISTERED:
+#ifndef NDEBUG
+			// Generally unused, but ensure we don't do an implementation error
+			infostream << "Server tells us we sent a message on channel `" << channel
+				<< "` but we are not registered. Message was dropped." << std::endl;
+#endif
+			break;
+		case MODCHANNEL_SIGNAL_SET_STATE: {
+			u8 state;
+			*pkt >> state;
+
+			if (state == MODCHANNEL_STATE_INIT || state >= MODCHANNEL_STATE_MAX) {
+				infostream << "Received wrong channel state " << state
+						<< ", ignoring." << std::endl;
+				return;
+			}
+
+			m_modchannel_mgr->setChannelState(channel, (ModChannelState) state);
+			infostream << "Server sets mod channel `" << channel
+					<< "` in read-only mode." << std::endl;
+			break;
+		}
+		default:
+#ifndef NDEBUG
+			warningstream << "Received unhandled mod channel signal ID "
+				<< signal << ", ignoring." << std::endl;
+#endif
+			valid_signal = false;
+			break;
+	}
+
+	// If signal is valid, forward it to client side mods
+	if (valid_signal)
+		m_script->on_modchannel_signal(channel, signal);
+}
