@@ -17,18 +17,17 @@ with this program; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */
 
-#ifndef PROFILER_HEADER
-#define PROFILER_HEADER
+#pragma once
 
 #include "irrlichttypes.h"
+#include <cassert>
 #include <string>
 #include <map>
+#include <ostream>
 
-#include "threading/mutex.h"
 #include "threading/mutex_auto_lock.h"
 #include "util/timetaker.h"
 #include "util/numeric.h"      // paging()
-#include "debug.h"             // assert()
 
 #define MAX_PROFILER_TEXT_ROWS 20
 
@@ -43,9 +42,7 @@ extern Profiler *g_profiler;
 class Profiler
 {
 public:
-	Profiler()
-	{
-	}
+	Profiler() = default;
 
 	void add(const std::string &name, float value)
 	{
@@ -83,11 +80,8 @@ public:
 	void clear()
 	{
 		MutexAutoLock lock(m_mutex);
-		for(std::map<std::string, float>::iterator
-				i = m_data.begin();
-				i != m_data.end(); ++i)
-		{
-			i->second = 0;
+		for (auto &it : m_data) {
+			it.second = 0;
 		}
 		m_avgcounts.clear();
 	}
@@ -177,7 +171,7 @@ public:
 	}
 
 private:
-	Mutex m_mutex;
+	std::mutex m_mutex;
 	std::map<std::string, float> m_data;
 	std::map<std::string, int> m_avgcounts;
 	std::map<std::string, float> m_graphvalues;
@@ -196,11 +190,8 @@ public:
 			ScopeProfilerType type = SPT_ADD);
 	~ScopeProfiler();
 private:
-	Profiler *m_profiler;
+	Profiler *m_profiler = nullptr;
 	std::string m_name;
-	TimeTaker *m_timer;
+	TimeTaker *m_timer = nullptr;
 	enum ScopeProfilerType m_type;
 };
-
-#endif
-
