@@ -30,7 +30,7 @@ extern "C" {
 #include "constants.h"
 
 
-#define CHECK_TYPE(index, name, type) do { \
+#define CHECK_TYPE(index, name, type) { \
 		int t = lua_type(L, (index)); \
 		if (t != (type)) { \
 			std::string traceback = script_get_backtrace(L); \
@@ -38,7 +38,7 @@ extern "C" {
 				" (expected " + lua_typename(L, (type)) + \
 				" got " + lua_typename(L, t) + ").\n" + traceback); \
 		} \
-	} while(0)
+	}
 #define CHECK_POS_COORD(name) CHECK_TYPE(-1, "position coordinate '" name "'", LUA_TNUMBER)
 #define CHECK_FLOAT_RANGE(value, name) \
 if (value < F1000_MIN || value > F1000_MAX) { \
@@ -51,6 +51,15 @@ if (value < F1000_MIN || value > F1000_MAX) { \
 #define CHECK_POS_TAB(index) CHECK_TYPE(index, "position", LUA_TTABLE)
 
 
+void push_float_string(lua_State *L, float value)
+{
+	std::stringstream ss;
+	std::string str;
+	ss << value;
+	str = ss.str();
+	lua_pushstring(L, str.c_str());
+}
+
 void push_v3f(lua_State *L, v3f p)
 {
 	lua_newtable(L);
@@ -71,6 +80,26 @@ void push_v2f(lua_State *L, v2f p)
 	lua_setfield(L, -2, "y");
 }
 
+void push_v3_float_string(lua_State *L, v3f p)
+{
+	lua_newtable(L);
+	push_float_string(L, p.X);
+	lua_setfield(L, -2, "x");
+	push_float_string(L, p.Y);
+	lua_setfield(L, -2, "y");
+	push_float_string(L, p.Z);
+	lua_setfield(L, -2, "z");
+}
+
+void push_v2_float_string(lua_State *L, v2f p)
+{
+	lua_newtable(L);
+	push_float_string(L, p.X);
+	lua_setfield(L, -2, "x");
+	push_float_string(L, p.Y);
+	lua_setfield(L, -2, "y");
+}
+
 v2s16 read_v2s16(lua_State *L, int index)
 {
 	v2s16 p;
@@ -84,21 +113,6 @@ v2s16 read_v2s16(lua_State *L, int index)
 	return p;
 }
 
-v2s16 check_v2s16(lua_State *L, int index)
-{
-	v2s16 p;
-	CHECK_POS_TAB(index);
-	lua_getfield(L, index, "x");
-	CHECK_POS_COORD("x");
-	p.X = lua_tonumber(L, -1);
-	lua_pop(L, 1);
-	lua_getfield(L, index, "y");
-	CHECK_POS_COORD("y");
-	p.Y = lua_tonumber(L, -1);
-	lua_pop(L, 1);
-	return p;
-}
-
 void push_v2s16(lua_State *L, v2s16 p)
 {
 	lua_newtable(L);
@@ -344,6 +358,7 @@ aabb3f read_aabb3f(lua_State *L, int index, f32 scale)
 		box.MaxEdge.Z = lua_tonumber(L, -1) * scale;
 		lua_pop(L, 1);
 	}
+	box.repair();
 	return box;
 }
 
@@ -437,58 +452,6 @@ bool getstringfield(lua_State *L, int table,
 	return got;
 }
 
-bool getintfield(lua_State *L, int table,
-		const char *fieldname, int &result)
-{
-	lua_getfield(L, table, fieldname);
-	bool got = false;
-	if(lua_isnumber(L, -1)){
-		result = lua_tointeger(L, -1);
-		got = true;
-	}
-	lua_pop(L, 1);
-	return got;
-}
-
-bool getintfield(lua_State *L, int table,
-		const char *fieldname, u8 &result)
-{
-	lua_getfield(L, table, fieldname);
-	bool got = false;
-	if(lua_isnumber(L, -1)){
-		result = lua_tointeger(L, -1);
-		got = true;
-	}
-	lua_pop(L, 1);
-	return got;
-}
-
-bool getintfield(lua_State *L, int table,
-		const char *fieldname, u16 &result)
-{
-	lua_getfield(L, table, fieldname);
-	bool got = false;
-	if(lua_isnumber(L, -1)){
-		result = lua_tointeger(L, -1);
-		got = true;
-	}
-	lua_pop(L, 1);
-	return got;
-}
-
-bool getintfield(lua_State *L, int table,
-		const char *fieldname, u32 &result)
-{
-	lua_getfield(L, table, fieldname);
-	bool got = false;
-	if(lua_isnumber(L, -1)){
-		result = lua_tointeger(L, -1);
-		got = true;
-	}
-	lua_pop(L, 1);
-	return got;
-}
-
 bool getfloatfield(lua_State *L, int table,
 		const char *fieldname, float &result)
 {
@@ -569,6 +532,13 @@ bool getboolfield_default(lua_State *L, int table,
 	return result;
 }
 
+v3s16 getv3s16field_default(lua_State *L, int table,
+		const char *fieldname, v3s16 default_)
+{
+	getv3intfield(L, table, fieldname, default_);
+	return default_;
+}
+
 void setstringfield(lua_State *L, int table,
 		const char *fieldname, const char *value)
 {
