@@ -169,24 +169,6 @@ void ScriptApiEntity::luaentity_GetProperties(u16 id,
 	// Set default values that differ from ObjectProperties defaults
 	prop->hp_max = 10;
 
-	/* Read stuff */
-
-	prop->hp_max = getintfield_default(L, -1, "hp_max", 10);
-
-	getboolfield(L, -1, "physical", prop->physical);
-	getboolfield(L, -1, "collide_with_objects", prop->collideWithObjects);
-
-	getfloatfield(L, -1, "weight", prop->weight);
-
-	lua_getfield(L, -1, "collisionbox");
-	if (lua_istable(L, -1))
-		prop->collisionbox = read_aabb3f(L, -1, 1.0);
-	lua_pop(L, 1);
-
-	getstringfield(L, -1, "visual", prop->visual);
-
-	getstringfield(L, -1, "mesh", prop->mesh);
-
 	// Deprecated: read object properties directly
 	read_object_properties(L, -1, prop, getServer()->idef());
 
@@ -257,19 +239,17 @@ bool ScriptApiEntity::luaentity_Punch(u16 id,
 	setOriginFromTable(object);
 	PCALL_RES(lua_pcall(L, 6, 1, error_handler));
 
-	bool retval = lua_toboolean(L, -1);
+	bool retval = readParam<bool>(L, -1);
 	lua_pop(L, 2); // Pop object and error handler
 	return retval;
 }
 
-// Calls entity:on_rightclick(ObjectRef clicker)
-void ScriptApiEntity::luaentity_Rightclick(u16 id,
-		ServerActiveObject *clicker)
+// Calls entity[field](ObjectRef self, ObjectRef sao)
+bool ScriptApiEntity::luaentity_run_simple_callback(u16 id,
+	ServerActiveObject *sao, const char *field)
 {
 	SCRIPTAPI_PRECHECKHEADER
 
-	//infostream<<"scriptapi_luaentity_step: id="<<id<<std::endl;
-
 	int error_handler = PUSH_ERROR_HANDLER(L);
 
 	// Get core.luaentities[id]
@@ -277,18 +257,45 @@ void ScriptApiEntity::luaentity_Rightclick(u16 id,
 	int object = lua_gettop(L);
 	// State: object is at top of stack
 	// Get function
-	lua_getfield(L, -1, "on_rightclick");
+	lua_getfield(L, -1, field);
 	if (lua_isnil(L, -1)) {
-		lua_pop(L, 2); // Pop on_rightclick and entity
-		return;
+		lua_pop(L, 2); // Pop callback field and entity
+		return false;
 	}
 	luaL_checktype(L, -1, LUA_TFUNCTION);
-	lua_pushvalue(L, object); // self
-	objectrefGetOrCreate(L, clicker); // Clicker reference
+	lua_pushvalue(L, object);  // self
+	objectrefGetOrCreate(L, sao);  // killer reference
 
 	setOriginFromTable(object);
-	PCALL_RES(lua_pcall(L, 2, 0, error_handler));
+	PCALL_RES(lua_pcall(L, 2, 1, error_handler));
 
+	bool retval = readParam<bool>(L, -1);
 	lua_pop(L, 2); // Pop object and error handler
+	return retval;
 }
 
+bool ScriptApiEntity::luaentity_on_death(u16 id, ServerActiveObject *killer)
+{
+	return luaentity_run_simple_callback(id, killer, "on_death");
+}
+
+// Calls entity:on_rightclick(ObjectRef clicker)
+void ScriptApiEntity::luaentity_Rightclick(u16 id, ServerActiveObject *clicker)
+{
+	luaentity_run_simple_callback(id, clicker, "on_rightclick");
+}
+
+void ScriptApiEntity::luaentity_on_attach_child(u16 id, ServerActiveObject *child)
+{
+	luaentity_run_simple_callback(id, child, "on_attach_child");
+}
+
+void ScriptApiEntity::luaentity_on_detach_child(u16 id, ServerActiveObject *child)
+{
+	luaentity_run_simple_callback(id, child, "on_detach_child");
+}
+
+void ScriptApiEntity::luaentity_on_detach(u16 id, ServerActiveObject *parent)
+{
+	luaentity_run_simple_callback(id, parent, "on_detach");
+}
