@@ -1,8 +1,8 @@
 #! /bin/bash
 function perform_lint() {
 	echo "Performing LINT..."
-	if hash clang-format-3.9 2>/dev/null; then
-		CLANG_FORMAT=clang-format-3.9
+	if hash clang-format-5.0 2>/dev/null; then
+		CLANG_FORMAT=clang-format-5.0
 	else
 		CLANG_FORMAT=clang-format
 	fi
@@ -20,7 +20,7 @@ function perform_lint() {
 			whitelisted=$(awk '$1 == "'$f'" { print 1 }' "$CLANG_FORMAT_WHITELIST")
 
 			# If file is not whitelisted, mark a failure
-			if [ -z ${whitelisted} ]; then
+			if [ -z "${whitelisted}" ]; then
 				errorcount=$((errorcount+1))
 
 				printf "The file %s is not compliant with the coding style" "$f"
